@@ -8,12 +8,12 @@ import six
 import unittest
 
 from mock import patch
-from . import TembaClient
+from . import __version__
+from .client import TembaClient
 from .base import TembaObject, SimpleField, IntegerField, DatetimeField, ObjectListField, TembaException
 from .base import TembaNoSuchObjectError, TembaMultipleResultsError, TembaAPIError, TembaConnectionError
-from .types import Group, Broadcast
+from .types import Broadcast, Group, FlowDefinition
 from .utils import format_iso8601, parse_iso8601
-from .version import __version__
 
 
 class MockResponse(object):
@@ -158,15 +158,15 @@ class TembaClientTest(unittest.TestCase):
         self.assertEqual(field.label, "Chat Name")
         self.assertEqual(field.value_type, 'T')
 
-    def test_create_flow(self, mock_request):
-        mock_request.return_value = MockResponse(200, _read_json('flows_created'))
-        flow = self.client.create_flow("Ping", 'F')
+        # with key provided
+        field = self.client.create_field("Chat Name", 'T', key='chat_name')
 
-        expected_body = {'name': "Ping", 'flow_type': 'F'}
-        self.assert_request(mock_request, 'post', 'flows', data=expected_body)
+        expected_body = {'label': "Chat Name", 'value_type': 'T', 'key': 'chat_name'}
+        self.assert_request(mock_request, 'post', 'fields', data=expected_body)
 
-        self.assertEqual(flow.uuid, 'a68567fa-ad95-45fc-b5f7-3ce90ebbd46d')
-        self.assertEqual(flow.name, "Ping")
+        self.assertEqual(field.key, 'chat_name')
+        self.assertEqual(field.label, "Chat Name")
+        self.assertEqual(field.value_type, 'T')
 
     def test_create_label(self, mock_request):
         mock_request.return_value = MockResponse(200, _read_json('labels_created'))
@@ -651,8 +651,7 @@ class TembaClientTest(unittest.TestCase):
                                  before=datetime.datetime(2014, 12, 12, 22, 34, 36, 123000, pytz.utc),
                                  after=datetime.datetime(2014, 12, 12, 22, 34, 36, 234000, pytz.utc),
                                  text='heists',
-                                 archived=False,
-                                 reverse=True)
+                                 archived=False)
 
         self.assert_request(mock_request, 'get', 'messages', params={'id': [123, 234],
                                                                      'broadcast': [345, 456],
@@ -661,8 +660,7 @@ class TembaClientTest(unittest.TestCase):
                                                                      'before': '2014-12-12T22:34:36.123000',
                                                                      'after': '2014-12-12T22:34:36.234000',
                                                                      'text': 'heists',
-                                                                     'archived': 0,
-                                                                     'reverse': 1})
+                                                                     'archived': 0})
 
         # test getting by broadcast object (should extract id)
         broadcast = Broadcast.create(id=123)
@@ -777,6 +775,23 @@ class TembaClientTest(unittest.TestCase):
                          'action': 'remove', 'group': 'Testers'}
         self.assert_request(mock_request, 'post', 'contact_actions', data=expected_body)
 
+    def test_save_flow_definition(self, mock_request):
+        mock_request.return_value = MockResponse(200, _read_json('flow_definition_created'))
+
+        definition = FlowDefinition.create(metadata={'name': "Empty Flow"},
+                                           version=7,
+                                           base_language='eng',
+                                           flow_type='F',
+                                           action_sets=[],
+                                           rule_sets=[],
+                                           entry=None)
+        self.client.save_flow_definition(definition)
+
+        expected_body = {'metadata': {'name': "Empty Flow"}, 'version': 7,
+                         'base_language': 'eng', 'flow_type': 'F',
+                         'action_sets': [], 'rule_sets': [], 'entry': None}
+        self.assert_request(mock_request, 'post', 'flow_definition', data=expected_body)
+
     def test_unarchive_messages(self, mock_request):
         mock_request.return_value = MockResponse(204)
         self.client.unarchive_messages(messages=[123, 234, 345])
@@ -824,18 +839,6 @@ class TembaClientTest(unittest.TestCase):
         self.assertEqual(contact.language, None)
         self.assertEqual(contact.modified_on, datetime.datetime(2014, 10, 1, 6, 54, 9, 817000, pytz.utc))
 
-    def test_update_flow(self, mock_request):
-        mock_request.return_value = MockResponse(200, _read_json('flows_created'))
-        flow = self.client.update_flow('a68567fa-ad95-45fc-b5f7-3ce90ebbd46d', "Ping", 'F')
-
-        expected_body = {'uuid': 'a68567fa-ad95-45fc-b5f7-3ce90ebbd46d',
-                         'name': "Ping",
-                         'flow_type': 'F'}
-        self.assert_request(mock_request, 'post', 'flows', data=expected_body)
-
-        self.assertEqual(flow.uuid, 'a68567fa-ad95-45fc-b5f7-3ce90ebbd46d')
-        self.assertEqual(flow.name, "Ping")
-
     def test_update_label(self, mock_request):
         mock_request.return_value = MockResponse(200, _read_json('labels_created'))
         label = self.client.update_label('affa6685-0725-49c7-a15a-96f301d996e4', "Really High Priority")
@@ -865,8 +868,8 @@ class TembaClientTest(unittest.TestCase):
         try:
             self.client.update_label('12345678', "Really High Priority")
         except TembaAPIError as ex:
-            self.assertEqual(ex.errors, {})
-            self.assertEqual(six.text_type(ex), "API request error. Caused by: 400 Client Error: ...")
+            self.assertEqual(ex.errors, {'non_field_errors': ['xyz']})
+            self.assertEqual(six.text_type(ex), "API request error. Caused by: xyz")
         else:
             self.fail("Should have thrown exception")
 
@@ -949,6 +952,13 @@ class TembaObjectTest(unittest.TestCase):
         self.assertRaises(TembaException, TestType.deserialize,
                           {'foo': 'a', 'bar': 'x', 'doh': '2014-01-02T03:04:05', 'hum': {}})
 
+    def test_serialize(self):
+        obj = TestType.create(foo='a', bar=123, doh=datetime.datetime(2014, 1, 2, 3, 4, 5, 0, pytz.UTC),
+                              hum=[TestSubType.create(zed='b')])
+
+        json_obj = obj.serialize()
+        self.assertEqual(json_obj, {'foo': 'a', 'bar': 123, 'doh': '2014-01-02T03:04:05.000000', 'hum': [{'zed': 'b'}]})
+
 
 def _read_json(filename):
     handle = open('test_files/%s.json' % filename)
